
/*
   Dieses Programm enthlt ein einfaches Beispiel fr die Programmierung
   von HotHelp-Handlern. Es wird ein Handler installiert, der die bliche
   Meldung beim Nicht-Auffinden eines gewnschten Schlssels durch eine
   ausfhrlichere Nachricht ersetzt.
*/

#include "HotHelp.h"
#include <Functions.h>
#include <StdIO.h>
#include <String.h>


LONG        HotHelpHandler (struct HH_HandlerData *);
/*
   Die folgenden drei Zeilen sind spezifisch fr den Aztec-C. ber das
   #pragma wird definiert, da der einzige Parameter der HotHelpHandler-
   Funktion nicht auf dem Stack, sondern in Register A0 bergeben wird.
   'int_start ()' und 'int_end ()' initialisieren einige Register, da
   die Funktion nicht als Teil des Hauptprogramms ausgefhrt wird.
*/
#pragma     regcall        (HotHelpHandler (A0))
VOID        int_start      (VOID);
VOID        int_end        (VOID);


struct IntuitionBase *  IntuitionBase;
struct HotHelpBase *    HotHelpBase;
/*
   Die Struktur wird zum Anmelden des Handlers verwendet.
*/
struct Interrupt        inter;
/*
   Diese beiden Strukturen werden benutzt, um einen Text zur Darstellung
   an HotHelp zu bergeben.
*/
struct HH_TranslateData hh_trans;
struct HH_CustomText    hh_ctext;
/*
   Dieses Feld nimmt den darzustellenden Text auf.
*/
BYTE                    buf [1000];


VOID main (VOID)
   {
   if (IntuitionBase = OpenLibrary ("intuition.library", 33))
      {
      if (HotHelpBase = OpenLibrary ("hothelp.library", 3))
         {
         /*
            Die Funktion 'HotHelpHandler' wird als Handler angemeldet.
            Besondere Daten werden nicht bermittelt und der Handler
            luft auf der niedrigsten Prioritt - er wird also als
            letzter von allen installierten Handlern angesprochen.
         */
         inter.is_Code        = HotHelpHandler;
         inter.is_Data        = NULL;
         inter.is_Node.ln_Pri = -128;

         /*
            Der Handler wird installiert. Anschlieend warten wir dann
            auf den Abbruch das Programms ber CTRL-C, CTRL-D, CTRL-E
            oder CTRL-F und entfernen den Handler wieder.
         */
         HH_AddHandler (&inter);
         Wait (SIGBREAKF_CTRL_C | SIGBREAKF_CTRL_D |
               SIGBREAKF_CTRL_E | SIGBREAKF_CTRL_F);
         HH_RemHandler (&inter);

         CloseLibrary (HotHelpBase);
         }
      else
         printf ("Fehler bei HotHelpBase\n");
      CloseLibrary (IntuitionBase);
      }
   else
      printf ("Fehler bei IntuitionBase\n");
   }


/*
   Diese Funktion wird nun von HotHelp jedesmal aufgerufen, wenn ein
   unbekannter Schlssel-Begriff vorliegt. HotHelp spricht dann der
   Reihe nach alle Handler an; nur, wenn auch keiner der Handler etwas
   mit dem Begriff anfangen kann, landet er bei uns (wir sind mit
   Prioritt -128 ganz am Ende der Liste). Wir zeigen dann einfach
   eine nette Fehlermeldung an, die etwas ausfhrlicher ist als das
   bliche "Zu dem gesuchten Begriff konnten keine passenden Schlssel
   gefunden werden!"
*/
LONG HotHelpHandler
   (
   struct HH_HandlerData * hdata
   )

   {
   /*
      Unbekannte Kommandos, die nicht im switch auftauchen, werden stets
      mit FALSE beantwortet.
   */
   LONG                    ret   = FALSE;

   /*
      Diese Funktion des Aztec-C-Compilers sichert einige Register auf
      dem Stapel und initialisiert sie mit den bentigten Werten fr
      das Programm (z.B. ist A4 ein Zeiger auf die Daten des Programms).
      Wenn Sie mit einem anderen Compiler arbeiten, mssen Sie hier die
      entsprechende Funktion Ihres Compilers einsetzen.
   */
   int_start ();

   switch (hdata->action)
      {
      case HHA_PROJECTS:
         /*
            Dieser einfache Handler untersttzt keine Projekt-Namen, so
            da wir einfach TRUE zurckgeben.
         */
         ret = TRUE;
         break;
      case HHA_KEY:
         /*
            Wenn wir hier ankommen, ist folgendes passiert: Es wurde ein
            Schlssel angefordert, der nicht in den Standard-Projekten
            enthalten war. Anschlieend wurden alle Handler hherer
            Prioritt angesprochen, die ebenfalls keinen passenden Text
            verfgbar haben. Wir knnen also davon ausgehen, da es keinen
            passenden Text gibt.
            In diesem Fall wrde HotHelp normalerweise die etwas
            trockene Meldung "Zu dem gesuchten Begriff konnten keine
            passenden Schlssel gefunden werden!" ausgeben - aber jetzt
            sind ja noch wir an der Reihe!
         */
         if (hdata->pro_pattern || hdata->key_pattern)
            {
            /*
               Wenn der Projekt- oder Schlssel-Begriff ein Muster-Zeichen
               beinhaltet, knnen wir ihn nicht ausgeben (der String kann
               dann je nach der eingestellten Art des Pattern-Matchings
               eine bunte Mischung von Binrcodes beinhalten).
            */
            strcpy (buf, "\nEs tut mir wirklich sehr leid, aber ich "
                         "verfge ber keinen Text, der zu dem "
                         "eingegebenen Muster passen wrde (bitte "
                         "untertnigst um Entschuldigung).\n\n\n"
                         "(Diese devote Meldung wird Ihnen prsentiert "
                         "von |SCHR-7Beispiel6.c|SCHR-0)");
            }
         else
            {
            /*
               Weder Projekt- noch Schlssel-Begriff enthalten eines der
               Muster-Zeichen, so da wir die beiden Strings hier noch mit
               ausgeben knnen (macht sich doch irgendwie besser, oder?)
            */
            sprintf (buf, "\nEs tut mir wirklich sehr leid, aber ich "
                          "verfge ber keinen Text im Projekt |SCHR-3%s"
                          "|SCHR-0, der unter dem Schlssel-Begriff "
                          "|SCHR-3%s|SCHR-0 aufzufinden wre... (bitte "
                          "untertnigst um Entschuldigung).\n\n\n"
                          "(Diese devote Meldung wird Ihnen prsentiert "
                          "von |SCHR-7Beispiel6.c|SCHR-0)",
                          hdata->project, hdata->key);

            }

         /*
            Jetzt haben wir in 'buf' einen String, den wir im HotHelp-
            Fenster anzeigen mchten. Dazu mu er zuerst in HotHelps Text-
            Format bersetzt werden.
            Bei der bersetzung geben wir als Schriftart-Marke "|SCHR-" an;
            dadurch knnen wir im Text beliebige Bereiche mit verschiedenen
            TextStyles anzeigen.
            Als Absatzende-Marke geben wir "|ABS| an. Diese Kennung taucht
            zwar nirgendwo in unserem String auf; wrden wir sie aber nicht
            angeben, wrde der bersetzer auch keinen Flietext erzeugen!
            Die Tabulator-Gre wird auf 3 gesetzt (es wurden zwar keine
            Tabs verwendet, aber man wei ja nie...). Kompression wird
            ausgeschaltet.
         */
         hh_trans.version        = HH_VERSION;
         hh_trans.ascii_text     = buf;
         hh_trans.ascii_text_len = strlen (buf);
         hh_trans.mark_style     = "|SCHR-";
         hh_trans.mark_para      = "|ABS|";
         hh_trans.tab_size       = 3;
         hh_trans.compress       = FALSE;

         /*
            Diese Struktur nimmt den bersetzten Text auf; hier wird
            lediglich die Versions-Nummer initialisiert.
         */
         hh_ctext.version        = HH_VERSION;

         /*
            Nun bersetzen wir den Text. Geht dabei etwas schief, wird
            die 'error'-Variable in den passenden Rckgabewert
            umgewandelt; andernfalls geben wir TRUE zurck und bergeben
            HotHelp den Zeiger auf die ausgefllte HH_CustomText-Struktur
            mit unserem Text.
         */
         if (HH_Translate (&hh_trans, &hh_ctext))
            {
            ret = TRUE;
            hdata->custom_text = &hh_ctext;
            }
         else if (hh_trans.error == HHT_ERROR_MEM)
            ret = -1;
         else
            ret = -3;
         break;
      case HHA_FREE:
         /*
            Dieses Kommando erhalten wir immer als nchstes, nachdem ein
            HHA_KEY-Kommando erfolgreich bearbeitet wurde; hier knnen wir
            alles freigeben, was beim Erzeugen des Textes evtl. angelegt
            wurde. In unserem Fall mssen wir nur die von
            'HH_Translate ()' allozierten Bereiche wieder freigeben.
         */
         HH_TranslateFree (&hh_trans);
         break;
      }

   /*
      Setzt die oben ber 'int_start ()' genderten Register wieder
      zurck. Auch diese Funktion ist spezifisch fr Aztec-C und mu
      ggf. ausgetauscht werden.
   */
   int_end ();

   return (ret);
   }


